

service system/coldplug/input {
    depends = system/modules
    use = system/coldplug/pci system/coldplug/usb system/static-modules
    #start = /etc/hotplug/input.rc
    #start_args = start
    start {
#!/bin/sh

if [ ! -r /proc/bus/input/devices ]; then
    echo $"** can't synthesize input events - $PROCDIR/devices missing"
    return
fi

ACTION=add

input_reset_state () {
PRODUCT=
NAME=
PHYS=
EV=
KEY=
REL=
ABS=
MSC=
LED=
SND=
FF=
}

input_reset_state
export ACTION PRODUCT NAME PHYS EV KEY REL ABS MSC LED SND FF

#
# the following reads from /proc/bus/input/devices. It is inherently
# racy (esp. as this file may be changed by input.agent invocation)
# but so far input devices do not appear in sysfs
#
while read line; do
case "$line" in
    I:* )	# product ID
	eval "${line#I: }"
	PRODUCT="$Bus/$Vendor/$Product/$Version"
    ;;
    N:* )	# name
	eval "${line#N: }"
	NAME="$Name"
    ;;
    P:* )	# Physical
	eval "${line#P: }"
	PHYS="$Phys"
    ;;
    B:* )	# Controls supported
	line="${line#B: }"
	eval "${line%%=*}=\"${line#*=}\""
    ;;
    "" )	# End of block
	#echo "NAME=$NAME"
	/etc/hotplug/input.agent < /dev/null &
	input_reset_state
    ;;
esac
done < /proc/bus/input/devices

wait
exit 0
    }
}


service system/coldplug/pci {
    depends = system/initial system/modules 
    use = system/static-modules
start {
#!/bin/bash
    # make sure the pci agent will run
    ACTION=add
    PCI_CLASS=0
    PCI_ID=0:0
    PCI_SLOT=0:0.0
    PCI_SLOT_NAME=0:0.0
    PCI_SUBSYS_ID=0:0
    export ACTION PCI_CLASS PCI_ID PCI_SLOT PCI_SLOT_NAME PCI_SUBSYS_ID

	# 2.6 kernels only
	if [ -d /sys/bus/pci/devices/ ]; then
	    cd /sys/bus/pci/devices/
	    for PCI_DEVICE in *; do
		#echo "Found: $PCI_DEVICE"
		set `echo $PCI_DEVICE \
		    | sed -e 's/\([^:]*\):\(.*\):\(.*\)\.\(.*\)/\1 \2 \3 \4/'`
		PCI_SLOT_NAME=$2:$3.$4
		PCI_CLASS="`cat $PCI_DEVICE/class`"
		PCI_CLASS=${PCI_CLASS#0x}
		vendor_id=`cat $PCI_DEVICE/vendor`
		device_id=`cat $PCI_DEVICE/device`
		PCI_ID="${vendor_id#0x}:${device_id#0x}"
		sub_vendor_id=`cat $PCI_DEVICE/subsystem_vendor`
		sub_device_id=`cat $PCI_DEVICE/subsystem_device`
		PCI_SUBSYS_ID="${sub_vendor_id#0x}:${sub_device_id#0x}"
		#echo "debug: PCI_SLOT_NAME=$PCI_SLOT_NAME   PCI_CLASS=$PCI_CLASS  PCI_ID=$PCI_ID"
		/sbin/hotplug pci
	    done
	fi
    wait
    exit 0
}
}

# todo, add inside sctipt
service system/coldplug/pnp {
    depends = system/modules
    start = /etc/hotplug/pnp.rc
    start_args = start
}

# todo, add inside sctipt
service system/coldplug/isapnp {
    depends = system/modules
    start = /etc/hotplug/isapnp.rc
    start_args = start
}

service system/coldplug/usb {
    depends = system/initial system/checkroot system/modules system/usb
    use = system/coldplug/pci system/static-modules
    #start = /etc/hotplug/usb.rc
    #start_args = start
    start {
#!/bin/sh
ACTION=add
PRODUCT=0/0/0
TYPE=
INTERFACE=
DEVPATH=
DEVFS=/proc/bus/usb
DEVICE=
export ACTION PRODUCT TYPE INTERFACE DEVPATH DEVFS DEVICE

    if [ -d /sys/bus/usb/devices ]; then
		cd /sys/bus/usb/devices
		for device in /sys/bus/usb/devices/[0-9]*; do
		    devlink=$( readlink -f $device )
		    DEVPATH=${devlink#/sys}
		    if [ -f $devlink/../idVendor ]; then
			PRODUCT="$(cat $devlink/../idVendor)/$(cat $devlink/../idProduct)/$(cat $devlink/../bcdDevice)"
		        if [ -f $devlink/../../devnum ]
			then
			    devbus=$( ( echo -n 000 ; cat $devlink/../../devnum ) | grep -o ...\$ )
			    devdev=$( ( echo -n 000 ; cat $devlink/../devnum ) | grep -o ...\$ )
			    DEVICE="/proc/bus/usb/$devbus/$devdev"
			else
			    DEVICE=
			fi
			#echo "debug: $DEVICE"
			/etc/hotplug/usb.agent &
		    fi	
		done
    else
	echo "Usb support, seems not to be loaded or can't probe /sys/bus/usb/devices ..."
    fi
    wait
    exit 0
}

}

service system/coldplug {
    depends = system/coldplug/pci system/coldplug/usb system/coldplug/input
}

