
/* Initng, a next generation sysvinit replacement.
 * Copyright (C) 2005 Jimmy Wennlund <jimmy.wennlund@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#define _GNU_SOURCE
#include <fcntl.h>
#include <sys/file.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <signal.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <getopt.h>

/* #include "ngc.h" */
#include "../src/initng.h"
#include "../src/active_status.h"

#ifdef NGDC
#undef CONTROL_PATH
#define CONTROL_PATH TEST_CONTROL_PATH
#endif

/* Global in and out fifos */
static int send_pipe, recv_pipe;

/* functions */
int open_pipes(void);
int close_pipes(void);
unsigned int command_fetch_int(char *command);
int main(void);

/* open pipes to comunicate with */
int open_pipes(void)
{
    int i = 20;                 /* counter */

    /* open pipes */
    send_pipe = open(CONTROL_PATH "/in", O_WRONLY);
    recv_pipe = open(CONTROL_PATH "/out", O_RDONLY);

    /* check pipes */
    if (send_pipe < 0 || recv_pipe < 0)
    {
        printf("ngc: could not open " CONTROL_PATH "/in or " CONTROL_PATH
               "/out!\n");
        return 1;
    }

    /* TRY SET LOCK, so ther is not another program talking to initng */
    while (lockf(send_pipe, F_LOCK, 1))
    {
        printf("ngc: could not acquire lock!  trying .... %i...\n", i);
        i--;
        if (i == 0)
        {
            printf("ngc: error giving up!\n");
            return (1);
        }
        sleep(1);
    }
    return 0;
}

/* close pipes */
int close_pipes(void)
{
    return (close(send_pipe) + close(recv_pipe));
}

/* send command and return the fetched retun number */
unsigned int command_fetch_int(char *command)
{
    char got_from_read[20];

    if (open_pipes() == 1)
        return (-1);
    write(send_pipe, command, strlen(command));
    /* no idea to stress initng */
    usleep(50000);
    got_from_read[read(recv_pipe, &got_from_read, 19)] = '\0';
    if (close_pipes())
        return (-1);
    return (atoi(got_from_read));
}

/* THIS IS MAIN */
int main(void)
{
    int p = 0;
    int i;
    int last = 0;
    char **arg = NULL;

#ifdef VERBOSE
    /* print banner */
    printf("Next Generetion init, splash_update.\n");
    printf("written by Jimmy Wennlund <jimmy.wennlund@gmail.com\n\n");
#endif
    arg = (char **) calloc(7, sizeof(char *));
    arg[0] = "/sbin/splash_util";
    arg[1] = "--vc=0";
    arg[2] = calloc(10, sizeof(char));
    arg[3] = "--theme=default";
    arg[4] = "--mode=s";
    arg[5] = "--cmd=paint";
    arg[7] = NULL;

    /* do this all over again, until 95% completed */
    while ((p = command_fetch_int("n")) < 95)
    {

        /* if progress changed */
        if (p != last)
        {
            if (fork())
            {
#ifdef DEBUG
                printf("progress=%i\n", p * 655);
#endif
                wait(&i);
            }
            else
            {
                sprintf(arg[2], "--progress=%i", p * 655);
#ifdef DEBUG
                printf("%s\n", arg[2]);
#endif
                execve("/sbin/splash_util", arg, environ);
                _exit(1);
            }
            last = p;
        }
        /* wait a moment */
        usleep(100000);
    }

    arg[1] = "--cmd=setmode";
    arg[2] = "--mode=v";
    arg[3] = NULL;
    arg[4] = NULL;
    arg[5] = NULL;
    if (fork())
    {
        wait(&i);
#ifdef DEBUG
        printf("Splash remover exited.\n");
#endif
    }
    else
    {
        execve("/sbin/splash_util", arg, environ);
        _exit(1);
    }
    return 0;
}
