
/* Initng, a next generation sysvinit replacement.
 * Copyright (C) 2005 Jimmy Wennlund <jimmy.wennlund@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifdef BUSYBOX
#include "../../busybox-1.00/include/busybox.h"
#include "../../busybox-1.00/include/libbb.h"
#endif
#include "initng.h"

#include <time.h>                           /* time() */
#include <fcntl.h>                          /* fcntl() */
#include <unistd.h>                         /* execv() pipe() usleep() pause() chown() */
#include <sys/wait.h>                       /* waitpid() sa */
#include <sys/ioctl.h>                      /* ioctl() */
#include <stdlib.h>                         /* malloc() free() exit() */
#include <stdio.h>
#include <string.h>                         /* memset() strlen() */

#include "initng_execute.h"                 /* new_environ() */
#include "initng_global.h"
#include "bash_executor.h"
#include "main.h"

static void do_kill(int sig_num);

/*extern char **environ; */
pid_t pid_fork = 0;

static void do_kill(int sig_num)
{
    D_("do_kill(): #####################    Got signal: %i   Forwarding to %i!\n", sig_num, pid_fork);
    signal(sig_num, do_kill);
    kill(pid_fork, sig_num);
}

pid_t bash_this(char *bash_code, active_h * s)
{
    int pipe_fork[2];
    int status = 0;             /* waitpid status */

    D_("bash_this(%s);\n", s->name);
    pipe(pipe_fork);
    /*fcntl(pipe_fork[0],F_SETFD,FD_CLOEXEC); */
    /*fcntl(pipe_fork[1],F_SETFD,FD_CLOEXEC); */

    /* vfork is faster than fork, because it doesn't copy program memory, but orginal pid will be suspended
     * until fork reaches _exit or execve */

    switch (pid_fork = fork())
    {

        case -1:
            return (-1);
        case 0:                            /* child */
            D_("bash_this(): This is child: %i\n", pid_fork);
            dup2(pipe_fork[0], 0);          /* copy filedescriptor */
            close(pipe_fork[1]);
            /* fcntl(pipe_fork[0],F_SETFD,0); ??? */

#ifdef BUSYBOX
            {
                struct BB_applet *test;

                if ((test = find_applet_by_name("ash")))
                {

                    D_("bash_this(): Found, ash USING it\n");
                    _exit((*(test->main)) (0, NULL));

                }
                else
                {
                    F_("bash_this(): Not fount ASH, failing \n");
                    _exit(1);
                }
            }

#else
            {
                char **argtmp;
                argtmp = (char **) calloc(2, sizeof(char *));
                argtmp[0] =
                    (char *) calloc(1, sizeof(char) * (15 + strlen(s->name)));
                strcpy(argtmp[0], "bash_helper[");
                strcat(argtmp[0], s->name);
                strcat(argtmp[0], "]");
                argtmp[1] = NULL;
                execve("/bin/bash", argtmp, new_environ(s));
                free(argtmp[0]);
                free(argtmp);
            }
            F_("bash_this(): child die!\n EEEEEEEEEEEEEEEEEEEEEEERRRRRRRRRRRROOOOOOOOOORR!\n");
            /* exit 1, why???, to emit signal? */
            exit_initng(1);
            _exit(1);
#endif
        default:                           /* non fork comes here */
            /* if we need to wait for it */
            D_("bash_this(): This is parrent: %i\n", pid_fork);
            /* send these signal to child if we got them */
            close(pipe_fork[0]);
            signal(SIGTERM, do_kill);
            /* signal(SIGKILL, do_kill); kill is not set */

            write(pipe_fork[1], bash_code, strlen(bash_code));
            write(pipe_fork[1], "\n\nexit\n", 7);
            close(pipe_fork[1]);

            D_("bash_this(): TODO, tell nginit the pid of this process is %i!\n", pid_fork);

            wait(&status);

#ifdef DEBUG
            if (pid_fork > 0)
            {
                D_("bash_executor(): PID %i KILLED WITH:\n"
                   "WEXITSTATUS %i\n" "WIFEXITED %i\n" "WIFSIGNALED %i\n"
                   "WTERMSIG %i\n" "WCOREDUMP %i\n" "WIFSTOPPED %i\n"
                   "WSTOPSIG %i\n" "WIFCONTINUED %i\n\n", pid_fork,
                   WEXITSTATUS(status), WIFEXITED(status),
                   WIFSIGNALED(status), WTERMSIG(status),
                   WCOREDUMP(status), WIFSTOPPED(status),
                   WSTOPSIG(status), WIFCONTINUED(status));
            }
#endif

            return (WEXITSTATUS(status));

    }                                       /* End case */
}                                           /* end fork_and_exec() */
