
/* Initng, a next generation sysvinit replacement.
 * Copyright (C) 2005 Jimmy Wennlund <jimmy.wennlund@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#undef FILENAME
#define FILENAME "colorprint.c"
#include "initng.h"
#include "initng_handler.h"

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>

#include "initng_global.h"
#include "string_tools.h"
#include "colorprint.h"
#include "active_db.h"
#include "initng_history.h"
#include "main.h"

#define PE C_RED " %3i%% " C_OFF C_BLUE "%s" C_OFF MOVE_TO_R
#define P C_RED "      " C_OFF C_BLUE "%s" C_OFF MOVE_TO_R

static void out_restarting(char *name)
{
    printf(P "    [" C_GREEN "restart" C_OFF "]\n", name);
}

static void out_service_started(char *name, int pid)
{
    printf(PE "    [" C_GREEN "started" C_OFF "]       (pid : %i).\n",
           active_db_percent_started(), name, pid);
}

static void out_service_stopped(char *name)
{
    printf(P "    [" C_GREEN "stopped" C_OFF "]\n", name);
}

/*
   static void out_service_exited(char *name) {
   printf(P  "     [" C_GREEN "exited" C_OFF "]\n",name);
   }
 */
static void out_service_done(char *name)
{
    int time = 0;
    int percent = active_db_percent_started();

    if ((time = time_been_from(name, STARTING)))
        printf(PE "       [" C_GREEN "done" C_OFF
               "]   ( %i seconds to start. )\n", percent, name, time);
    else
        printf(PE "       [" C_GREEN "done" C_OFF "]\n", percent, name);
}

void out_system_up(void)
{
    printf("\n\n"
           "       runlevel \"%s\" up in, %i seconds.\n\n", g.runlevel,
           time_been_from(g.runlevel, LOADING));
}

static void out_service_starting(char *name)
{
    printf(PE "   [" C_GREEN "starting" C_OFF "]\n",
           active_db_percent_started(), name);
}

static void out_service_stopping(char *name)
{
    printf(P "   [" C_GREEN "stopping" C_OFF "]\n", name);
}

/*
   static void out_service_metstarting(char *name) {
   printf(P C_GREEN " dependecy met, now starting..."C_OFF"\n",name);
   }
 */

/*
   static void out_service_loading(char *name) {
   printf(P "    [" C_GREEN "loading" C_OFF "]\n",name);
   }
 */
void out_service_error(char *name, int r_code)
{
    printf(P "      [" C_RED "error" C_OFF "]      (ret_code: %i)\n", name,
           r_code);
}

void out_welcome(void)
{
    printf("\n" C_BLUE "     Next Generaton Init version ( ");
    printf(INITNG_VERSION);
    printf(" )" C_OFF);
    printf("\n" C_GREEN "     http://jw.dyndns.org/initng/\n" C_OFF);
    printf("     Author: Jimmy Wennlund <jimmy.wennlund@gmail.com>\n\n");
}

void out_all_services_exited(void)
{
    printf(C_RED "  All services exited.\n" C_OFF);
    printf("  Goodbye\n");
}

void print_output(active_h * service)
{
    if (!service)
        return;

    /* if, do some output and history adds */
    switch (service->a_status)
    {

        case STOPPED:
            out_service_stopped(service->name);
            return;
        case STARTING:
            out_service_starting(service->name);
            return;
        case DONE:
            out_service_done(service->name);
            return;
        case RUNNING:
            if (service->start_process && service->start_process->pid)
                out_service_started(service->name,
                                    service->start_process->pid);
            return;
        case STOP_MARKED:
            out_service_stopping(service->name);
            return;
        case RESPAWNING:
            out_restarting(service->name);
            return;
#ifdef DEBUG_EXTRA
        case WAITING_FOR_START_DEP:
            printf("waiting for dep to start %s.\n", service->name);
            return;
        case START_DEP_MET:
            printf("start dep met %s\n", service->name);
            /*out_service_metstarting(service->name); */
            return;
        case WAITING_FOR_STOP_DEP:
            printf("service %s is waiting for stop dep.\n", service->name);
            return;
        case STOP_DEP_MET:
            printf("service %s stop dep is met.\n", service->name);
            return;
        case STOPPING:
            printf("service %s stopping.\n", service->name);
            return;
        case NONE:
            printf("service %s *nostatus*.\n", service->name);
            return;
#endif
        default:
#ifdef DEBUG_EXTRA
            printf("Unknown status to set!\n");
#endif
            return;
    }
}
