
/* Initng, a next generation sysvinit replacement.
 * Copyright (C) 2005 Jimmy Wennlund <jimmy.wennlund@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "initng.h"

#include <time.h>                           /* time() */
#include <fcntl.h>                          /* fcntl() */
#include <unistd.h>                         /* execv() pipe() usleep() pause() chown() */
#include <sys/wait.h>                       /* waitpid() sa */
#include <sys/ioctl.h>                      /* ioctl() */
#include <stdlib.h>                         /* malloc() free() exit() */
#include <stdio.h>
#include <string.h>                         /* strdup() */
#include <assert.h>

#include "initng_global.h"
#include "process_db.h"
#include "main.h"
#include "string_tools.h"                   /* split() */
#include "initng_execute.h"
#include "bash_executor.h"
#include "active_db.h"

/*static void open_dev_console(); */
static void open_dev_out(active_h * srv);
static void initng_pause(void);
static pid_t initng_execute_fork_and_exec(process_h * process_to_exec,
                                          active_h * s);
static pid_t script_initng_execute_fork_and_exec(process_h * process_to_exec,
                                                 active_h * s);

/* this will create a process struct, and execute it */
int start_execute(active_h * service)
{
    assert(service);
    assert(service->from_service);
    D_("initng_execute(%s): There is a service->start file or script....\n",
       service->name);
    process_db_new(&service->start_process);

    /* P I D F I L E   H A C K */
    /* if we have to take pid form a pidfile, use pidfilehack */
    if (active_db_is(STRING_PID_FILE, service))
    {
        if (g.i_am_init)
            service->start_process->exec = strdup("/sbin/pidfilehack");
        else
            service->start_process->exec = strdup("/sbin/pidfiledhack");

        /*  pidfilehack <service> <pidfile> <daemon> <deamon_args> */
        service->start_process->exec_args = calloc(1, 200 * sizeof(char));
        strcpy(service->start_process->exec_args, service->name);
        strcat(service->start_process->exec_args, " ");
        strcat(service->start_process->exec_args,
               active_db_get_string(STRING_PID_FILE, service));
        strcat(service->start_process->exec_args, " ");
        strcat(service->start_process->exec_args,
               active_db_get_string(STRING_START, service));
        if (active_db_is(STRING_DAEMON_ARGS, service))
        {
            strcat(service->start_process->exec_args, " ");
            strcat(service->start_process->exec_args,
                   active_db_get_string(STRING_START_ARGS, service));
        }
    }
    else
    {
        service->start_process->exec =
            active_db_get_string(STRING_START, service);
        service->start_process->exec_args =
            active_db_get_string(STRING_START_ARGS, service);
    }

    D_("initng_execute(%s): LAUNCHING START OF %s \"%s\"\n",
       service->name, active_db_get_string(STRING_START, service),
       service->name);
    /* execute this */
    if (!initng_execute_fork_and_exec(service->start_process, service))
    {
        D_("initng_execute(%s): FAIL EXEUTING START initng_execute_fork_and_exec!\n", service->name);
        process_db_free(&service->start_process);
        return (FALSE);
    }

    /* if we use the pidfil hack, free the data duplicated */
    if (active_db_is(STRING_PID_FILE, service))
    {
        if (service->start_process->exec)
        {
            free(service->start_process->exec);
            service->start_process->exec = NULL;
        }
        if (service->start_process->exec_args)
        {
            free(service->start_process->exec_args);
            service->start_process->exec_args = NULL;
        }
    }

    return (TRUE);
}

/* this will create a process struct, and execute it */
int daemon_execute(active_h * service)
{
    assert(service);
    assert(service->from_service);
    D_("initng_execute(%s): There is a service->start file or script....\n",
       service->name);
    process_db_new(&service->start_process);

    /* P I D F I L E   H A C K */
    /* if we have to take pid form a pidfile, use pidfilehack */
    if (active_db_is(STRING_PID_FILE, service))
    {
        if (g.i_am_init)
            service->start_process->exec = strdup("/sbin/pidfilehack");
        else
            service->start_process->exec = strdup("/sbin/pidfiledhack");

        /*  pidfilehack <service> <pidfile> <daemon> <deamon_args> */
        service->start_process->exec_args = calloc(1, 200 * sizeof(char));
        strcpy(service->start_process->exec_args, service->name);
        strcat(service->start_process->exec_args, " ");
        strcat(service->start_process->exec_args,
               active_db_get_string(STRING_PID_FILE, service));
        strcat(service->start_process->exec_args, " ");
        strcat(service->start_process->exec_args,
               active_db_get_string(STRING_DAEMON, service));
        if (active_db_is(STRING_DAEMON_ARGS, service))
        {
            strcat(service->start_process->exec_args, " ");
            strcat(service->start_process->exec_args,
                   active_db_get_string(STRING_DAEMON_ARGS, service));
        }

    }
    else
    {
        service->start_process->exec =
            active_db_get_string(STRING_DAEMON, service);
        service->start_process->exec_args =
            active_db_get_string(STRING_DAEMON_ARGS, service);
    }

    D_("initng_execute(%s): LAUNCHING START OF %s \"%s\"\n",
       service->name, active_db_get_string(STRING_DAEMON, service),
       service->name);

    /* execute this */
    if (!initng_execute_fork_and_exec(service->start_process, service))
    {
        F_("initng_execute(%s): FAIL EXEUTING START initng_execute_fork_and_exec!\n", service->name);
        process_db_free(&service->start_process);
        return (FALSE);
    }

    /* if we use the pidfil hack, free the data duplicated */
    if (active_db_is(STRING_PID_FILE, service))
    {
        if (service->start_process->exec)
        {
            free(service->start_process->exec);
            service->start_process->exec = NULL;
        }
        if (service->start_process->exec_args)
        {
            free(service->start_process->exec_args);
            service->start_process->exec_args = NULL;
        }
    }

    return (TRUE);
}

/* will create a start process and execut it as a script */
int start_script_execute(active_h * service)
{
    assert(service);
    assert(service->from_service);
    process_db_new(&service->start_process);
    service->start_process->exec_args =
        active_db_get_string(STRING_START_SCRIPT, service);
    if (!script_initng_execute_fork_and_exec(service->start_process, service))
    {
        F_("initng_execute(%s): script_initng_execute_fork_and_exec failed!\n", service->name);
        process_db_free(&service->start_process);
        return (FALSE);
    }
    return (TRUE);
}

int stop_script_execute(active_h * service)
{
    assert(service);
    assert(service->from_service);
    process_db_new(&service->stop_process);
    service->stop_process->exec_args =
        active_db_get_string(STRING_STOP_SCRIPT, service);
    if (!script_initng_execute_fork_and_exec(service->stop_process, service))
    {
        F_("initng_execute(%s): script_initng_execute_fork_and_exec failed!\n", service->name);
        process_db_free(&service->stop_process);
        return (FALSE);
    }
    return (TRUE);
}

int stop_execute(active_h * service)
{
    assert(service);
    assert(service->from_service);
    D_("initng_execute(%s): There is a stop script/program....\n",
       service->name);
    process_db_new(&service->stop_process);

    service->stop_process->exec = active_db_get_string(STRING_STOP, service);
    service->stop_process->exec_args =
        active_db_get_string(STRING_STOP_ARGS, service);

    D_("initng_execute(%s): LAUNCHING STOP %s\n",
       service->name, service->stop_process->exec);

    /* execute this */
    if (!initng_execute_fork_and_exec(service->stop_process, service))
    {
        F_("initng_execute(%s): FAIL EXEUTING initng_execute_fork_and_exec!\n", service->name);
        process_db_free(&service->stop_process);
        return (FALSE);
    }

    return (TRUE);
}

/* creats an custom environ variable */
char **new_environ(active_h * s)
{
    int nr = 0;
    char **env;

    /* Better safe than sorry... */
    if (s && s->name == NULL)
        s->name = "unknown";

    /* count existing env,s */
#ifdef USE_OLD_ENV
    while (environ[nr])
        nr++;
#endif
#ifdef DEBUG
    if (s)
        D_("new_environ(): there is %i envs in config %s!\n",
           active_db_count_type(STRING_ENV, s), s->name);
#endif
    /*  Add 11 more environs */
    nr = nr + 11 + active_db_count_type(STRING_ENV, s);

    /* allocate */
    env = calloc(nr, sizeof(char *));

    /* duplicate */
#ifdef USE_OLD_ENV
    for (nr = 0; environ[nr]; nr++)
    {
        env[nr] = strdup(environ[nr]);
    }
#else
    nr = 0;
#endif
    /* add costum */
    env[nr++] = strdup("INITNG=" INITNG_VERSION);
    env[nr++] =
        strdup("PATH=/bin:/usr/bin:/sbin:/usr/sbin:/usr/local/bin:/opt/bin");
    env[nr++] = strdup("HOME=/root");
    env[nr++] = strdup("USER=root");
    env[nr++] = strdup("TERM=linux");
    /*env[nr++] = strdup("BASH_ENV=\"" INITNG_ROOT "/bash-env.sh\""); */

    if (s)
    {
        env[nr] = (char *) calloc(1, sizeof(char) * (9 + strlen(s->name)));
        strcpy(env[nr], "SERVICE=");
        strcat(env[nr], s->name);
        nr++;

        env[nr] = (char *) calloc(1, sizeof(char) * (6 + strlen(s->name)));
        strcpy(env[nr], "NAME=");
        strcat(env[nr], st_strip_path(s->name));
        nr++;

        env[nr] = (char *) calloc(1, sizeof(char) * (7 + strlen(s->name)));
        strcpy(env[nr], "CLASS=");
        strcat(env[nr], s->name);
        st_strip_end(&env[nr]);
        nr++;
    }

    /* insert all env strings form config */
    if ((env[nr] = active_db_get_next_string(STRING_ENV, s, NULL)))
    {
        nr++;
        while ((env[nr] =
                active_db_get_next_string(STRING_ENV, s, env[nr - 1])))
            nr++;
    }
    /* null last */
    if (env[nr] != NULL)
        env[nr++] = NULL;

#ifdef DEBUG
    for (nr = 0; env[nr]; nr++)
    {
        D_("new_environ(): environ[%i] = \"%s\"\n", nr, env[nr]);
    }
#endif

    /* return new environ */
    return (env);
}

/* this frees a environ variable */
void free_environ(char **tf)
{
    int i = 0;

    D_("free_environ();\n");
    assert(tf);
    for (i = 0; tf[i]; tf++)
    {
        free(tf[i]);
    }
    free(tf);
}

/* open console */

/*static void open_dev_console(void) {
   int fd;
   if ((fd=open("/dev/console",O_RDWR|O_NOCTTY))>=0) {
   dup2(fd,0);
   dup2(fd,1);
   dup2(fd,2);
   if (fd>2) close(fd);
   }
   } */

static void open_dev_out(active_h * srv)
{
    int fd1, fd2;
    char *stdout_def = STD_OUT;
    char *stderr_def = STD_ERR;

    char *stdout = NULL;
    char *stderr = NULL;

    assert(srv);

    /* get strings */
    stdout = active_db_get_string(STRING_STDOUT, srv);
    stderr = active_db_get_string(STRING_STDERR, srv);

    /* if stdout is set and stderr not, put stderr there to */
    if (stdout && !stderr)
        stderr = stdout;
    /*if(stderr && !stdout) stdout=stderr; */

    /* set defaults if not entered */
    if (!stdout)
        stdout = stdout_def;
    if (!stderr)
        stderr = stderr_def;

    /* if both stdout and stderr is same file */
    if (strcasecmp(stdout, stderr) == 0)
    {
        D_("open_dev_out(): Will open %s for stdout.\n", stdout);
        fd1 = open(stdout, O_RDWR | O_NOCTTY | O_CREAT | O_APPEND);
        fd2 = fd1;
    }
    else
    {
        D_("open_dev_out(): Will open %s for stdout and %s for stderr",
           stdout, stderr);
        fd1 = open(stdout, O_RDWR | O_NOCTTY | O_CREAT | O_APPEND);
        fd2 = open(stderr, O_RDWR | O_NOCTTY | O_CREAT | O_APPEND);
    }

    if (fd1 >= 0)
    {
        /* what are we gonna do with input? */
        dup2(fd1, 0);
        dup2(fd1, 1);
    }
    if (fd2 >= 0)
    {
        dup2(fd2, 2);
    }

    /* dont know about this one , make sure this is set. */
    if (fd1 > 2)
        close(fd1);
    if (fd2 > 2)
        close(fd2);

}

/* pause */
static void initng_pause(void)
{
    struct timespec req;

    req.tv_sec = 0;
    /*req.tv_nsec=500000000; */
    req.tv_nsec = 50000;
    nanosleep(&req, 0);
}

char *fix_variables(char *from, char *name)
{
    int i = 0;
    int j = 0;
    char *tmp;
    char *to;

    assert(from);
    assert(name);
    to = calloc(1, sizeof(char) * (strlen(from) + 150));
    while (from[i] != '\0')
    {
        if (from[i] == '$')
        {
            i++;
            if (strncmp(&from[i], "NAME", strlen("NAME")) == 0)
            {
                tmp = st_strip_path(name);
                strcpy(&to[j], tmp);
                i = i + strlen("NAME");
                j = j + strlen(tmp);
            }
            else if (strncmp(&from[i], "SERVICE", strlen("SERVICE")) == 0)
            {
                strcpy(&to[j], name);
                i = i + strlen("SERVICE");
                j = j + strlen(name);
            }
            else if (strncmp(&from[i], "CLASS", strlen("CLASS")) == 0)
            {
                tmp = st_get_path(name);
                strcpy(&to[j], tmp);
                i = i + strlen("CLASS");
                j = j + strlen(tmp);
                if (tmp)
                    free(tmp);
            }

        }
        else
        {
            to[j] = from[i];
            j++;
            i++;
        }
    }
    D_("fix_variables(): from \"%s\" to \"%s\".\n", from, to);
    return (to);
}

static pid_t
initng_execute_fork_and_exec(process_h * process_to_exec, active_h * s)
{

    /* called from inside the service directory, return the PID or 0 on error */

    /* This is the reale service kicker */
    pid_t pid_fork;             /* pid got from fork() */
    int try_count = 0;          /* Count tryings */
    char **argv = 0;            /* Arguments */
    int argc;                   /* number of arguments */
    char *dest = NULL;

    assert(process_to_exec);
    assert(process_to_exec->exec);

    /* Try to fork 3 times */
    while ((pid_fork = fork()) == -1)
    {
        if (try_count > 3)
            return 0;                       /* Already tried 3 times = no more try */
        sleep(++try_count * 2);             /* Sleep a while before trying agaim */
    }

    /* if this is child */
    if (pid_fork == 0)
    {

        if (g.i_am_init || active_db_is(STRING_STDOUT, s)
            || active_db_is(STRING_STDERR, s))
        {                                   /* If this is init */
            ioctl(0, TIOCNOTTY, 0);
            setsid();                       /* Run a program in a new session ??? */
            /*open_dev_console(); *//* open console for output */
            open_dev_out(s);                /* open console for output */
            /* ioctl(0, TIOCSCTTY, 1); */
            tcsetpgrp(0, getpgrp());        /* run this in foreground on fd 0 */
        }

        /* Take a break before launching */
        initng_pause();

        D_("FROM_FORK inting_execut_fork_and_exec(): deamon_this();\n");
        /* get args from service database */
        if (process_to_exec->exec_args)
        {
            D_("FROM_FORK initng_execute_fork_and_exec(%s): start args: %s\n",
               process_to_exec->exec, process_to_exec->exec_args);

            /* if we only vfork, we need to copy this string because split destroys it */
            /*temp=strdup(process_to_exec->exec_args); */
            dest = fix_variables(process_to_exec->exec_args, s->name);
            /* split args */
            argv = split_n(dest, &argc, 2, 1);

            /* i dont know about this */
            if (!argv[argc - 1])
            {
                argv[argc - 1] = '\0';      /* the last */
            }
            else
            {
                argv[argc] = '\0';          /* must be null */
            }
        }
        else
        {
            argv = calloc(2, sizeof(char *));   /* malloc a null */
            argv[1] = '\0';                 /* set to null */
        }

        /* set argv[0] */
        argv[0] = fix_variables(process_to_exec->exec, s->name);

        /* if stdin not points to standard */
        if (process_to_exec->stdin != 0)
        {
            dup2(process_to_exec->stdin, 0);    /* copy filedescriptor */
            fcntl(0, F_SETFD, 0);
        }

        /* and if stdout not points to usual */
        if (process_to_exec->stdout != 1)
        {
            dup2(process_to_exec->stdout, 1);   /* copy out stdout */
            dup2(process_to_exec->stdout, 2);   /* and stderr */
            fcntl(1, F_SETFD, 0);
            fcntl(2, F_SETFD, 0);
        }

        /* DISABLED FOR VALGRIND COMPLAINING REASON
         * {
         * int i;
         * for (i=3; i<1024; ++i) close(i);
         * } */

        /* finally execute */
        D_("FROM_FORK initng_execute_fork_and_exec(%s);\n",
           process_to_exec->exec);

#ifdef DEBUG
        {
            int i;

            for (i = 0; argv[i]; i++)
                D_("i.e.f.a.e.(): argv[%i]=\"%s\"\n", i, argv[i]);
        }
#endif

        /* FINALLY EXECUTE *//* execve replaces the running process */
        execve(argv[0], argv, new_environ(s));

        /* Will never get here if execve succeeded */
        F_("EEEEEEEEEEEEEEEEEEERRRRRRRRRRRRRRRRRRRRRRRRRRROOOOOOOOOOOOOOOOOOOOOOOOOORRRRRRRRRRRRRRR!!!!!!!!!\n");
        F_("Cant execute source %s, \"%s\" !\n", process_to_exec->exec,
           process_to_exec->exec_args);

        /* free variable, good for making valgrind quit */
        free(argv[0]);
        free(argv);
        free(dest);

        /* clean up */
        exit_initng(99);                    /* if 99 initng will only clean up */

        /* exit 1, why???, to emit signal? */
        _exit(1);
    }

    /* save pid of fork */
    D_("FROM_FORK Forkstarted pid %i.\n", pid_fork);
    process_to_exec->pid = pid_fork;
    return pid_fork;

    /* if to test want to lock this up util fork is done ...   waitpid(pid_fork,0,0); */
}                                           /* end fork_and_exec() */

static pid_t
script_initng_execute_fork_and_exec(process_h * process_to_exec, active_h * s)
{

    /* called from inside the service directory, return the PID or 0 on error */

    /* This is the real service kicker */
    pid_t pid_fork;             /* pid got from fork() */
    int try_count = 0;          /* Count tryings */
    int ret = 0;

    assert(process_to_exec);
    assert(process_to_exec->exec_args);

    /* Try to fork 3 times */
    while ((pid_fork = fork()) == -1)
    {
        if (try_count > 3)
            return 0;                       /* Already tried 3 times = no more try */
        sleep(++try_count * 2);             /* Sleep a while before trying again */
    }

    /* if this is child */
    if (pid_fork == 0)
    {

        if (g.i_am_init || active_db_is(STRING_STDOUT, s)
            || active_db_is(STRING_STDERR, s))
        {                                   /* If this is init */
            ioctl(0, TIOCNOTTY, 0);
            setsid();                       /* Run a program in a new session ??? */

            /*open_dev_console(); *//* open console for output */
            open_dev_out(s);
            /* ioctl(0, TIOCSCTTY, 1); */
            tcsetpgrp(0, getpgrp());        /* run this in foreground on fd 0 */
        }

        /* Take a break before launching */
        initng_pause();

        /* execute code */
        ret = bash_this(process_to_exec->exec_args, s);

        /* clean up */
        exit_initng(99);                    /* if 99 initng will only clean up */

        /* exit */
        _exit(ret);
    }

    /* save pid of fork */
    D_("FROM_FORK Forkstarted pid %i.\n", pid_fork);
    process_to_exec->pid = pid_fork;
    return pid_fork;

}
