
/* Initng, a next generatind sysvinit replacement.
 * Copyright (C) 2005 Jimmy Wennlund <jimmy.wennlund@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "initng.h"
#define _GNU_SOURCE
#include <stdio.h>
#include <alloca.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdlib.h>
#include <assert.h>

#define XML_EXT ".xml"
#include "string_tools.h"
#include "service_db.h"
#include "initng_global.h"
#include "initng_ezxml_parser.h"
#include "ezxml.h"
s_global g;

static int set_xml_parser(e_sdt type, char *value, service_h * from_service);
static int disable_xml_parser(e_sdt type, char *value,
                              service_h * from_service);
static int up_when_xml_parser(e_sdt type, char *value,
                              service_h * from_service);
static int string_xml_parser(e_sdt type, char *value,
                             service_h * from_service);
static int strings_xml_parser(e_sdt type, char *value,
                              service_h * from_service);

/*static int is_true(char *value); */
static int is_false(char *value);

typedef struct
{
    e_sdt type;
    const char *opt_name;
    int (*parse_func) (e_sdt t, char *value, service_h * f);
} option_xml_row;

static const option_xml_row option_xml_table[] = {
    {STRING_USE, "after", strings_xml_parser},
    {STRING_NEED_ALSO, "also_start", strings_xml_parser},
    {STRING_DAEMON_ARGS, "daemon_args", string_xml_parser},
    {STRING_DAEMON, "daemon", string_xml_parser},
    {STRING_NEED, "depends", strings_xml_parser},
    {STRING_NEED, "depend", strings_xml_parser},
    {STRING_NEED, "needs", strings_xml_parser},
    {STRING_NEED, "need", strings_xml_parser},
    {ENABLED_RESPAWN, "norespawn", disable_xml_parser},
    {STRING_USE, "uses", strings_xml_parser},
    {STRING_PROVIDE, "provides", strings_xml_parser},
    {STRING_PROVIDE, "provide", strings_xml_parser},
    {STRING_PID_FILE, "pid_file", string_xml_parser},
    {STRING_NEED, "requires", strings_xml_parser},
    {STRING_NEED, "require", strings_xml_parser},
    {ENABLED_RESPAWN, "respawn", set_xml_parser},
    {STRING_RDEPENDS, "rdepends", strings_xml_parser},
    {STRING_RDEPENDS, "rdepend", strings_xml_parser},
    {STRING_START_SCRIPT, "start_bash", string_xml_parser},
    {STRING_START_SCRIPT, "start_script", string_xml_parser},
    {STRING_START_SCRIPT, "startscript", string_xml_parser},
    {STRING_START_ARGS, "start_args", string_xml_parser},
    {STRING_START, "start", string_xml_parser},
    {STRING_STOP_SCRIPT, "stop_bash", string_xml_parser},
    {STRING_STOP_SCRIPT, "stop_script", string_xml_parser},
    {STRING_STOP_SCRIPT, "stopscript", string_xml_parser},
    {STRING_STOP_ARGS, "stop_args", string_xml_parser},
    {STRING_STOP, "stop", string_xml_parser},
    {EMPTY, "up_when", up_when_xml_parser},
    {STRING_USE, "use", strings_xml_parser},
    {EMPTY, "when_up", up_when_xml_parser},
    {STRING_STDOUT, "stdout", string_xml_parser},
    {STRING_STDOUT, "std_out", string_xml_parser},
    {STRING_STDERR, "stderr", string_xml_parser},
    {STRING_STDERR, "std_err", string_xml_parser},
    {STRING_ENV, "environ", strings_xml_parser},
    {STRING_ENV, "env", strings_xml_parser},

    {EMPTY, NULL, NULL}
};

static int set_xml_parser(e_sdt type, char *value, service_h * from_service)
{
    assert(from_service);
    if (!value)
        return (FALSE);
    D_("enable_parser(%i,%s,%s);\n", type, value, from_service->name);
    if (is_false(value))
    {
        service_db_remove(type, from_service);
    }
    else
    {
        /* else set true */
        service_db_set(type, from_service);
    }
    return (TRUE);
}

static int disable_xml_parser(e_sdt type, char *value,
                              service_h * from_service)
{
    assert(from_service);
    if (!value)
        return (FALSE);
    D_("disable_parser(%i,%s,%s);\n", type, value, from_service->name);

    if (is_false(value))
    {
        /* else set true */
        service_db_set(type, from_service);
    }
    return (TRUE);
}

/* special parse */
static int up_when_xml_parser(e_sdt type, char *value,
                              service_h * from_service)
{
    assert(from_service);
    type++;                                 /* do something, or compiler will fail */
    if (!value)
        return (FALSE);
    D_("up_when_parser(%i,%s,%s);\n", type, value, from_service->name);
    /* only one can be enabled, disable all others */
    service_db_remove(ENABLED_UP_WHEN_DONE, from_service);
    service_db_remove(ENABLED_UP_WHEN_PID_SET, from_service);
    service_db_remove(ENABLED_UP_WHEN_STARTED, from_service);

    if (strcasecmp(value, "started") == 0 || strcasecmp(value, "start") == 0)
        service_db_set(ENABLED_UP_WHEN_STARTED, from_service);

    else if (strcasecmp(value, "done") == 0 || strcasecmp(value, "exit") == 0)
        service_db_set(ENABLED_UP_WHEN_DONE, from_service);

    else if (strcasecmp(value, "pid_set") == 0 ||
             strcasecmp(value, "pidset") == 0 ||
             strcasecmp(value, "pid_is_set") == 0)
        service_db_set(ENABLED_UP_WHEN_PID_SET, from_service);

    else
    {
        F_("Fail parsing up_when (%s)\n", from_service->name);
        return (FALSE);
    }
    return (TRUE);
}

/* funtion copy full string into db */
static int string_xml_parser(e_sdt type, char *value,
                             service_h * from_service)
{
    assert(from_service);
    if (!value)
        return (FALSE);
    D_("string_parser(%i,%s,%s);\n", type, value, from_service->name);
    service_db_set_string(type, from_service, strdup(value));
    return (FALSE);
}

/* funtion splits up all string to single entrys */
static int strings_xml_parser(e_sdt type, char *value,
                              service_h * from_service)
{
    char *to = NULL;

    assert(from_service);
    if (!value)
        return (FALSE);
    D_("strings_xml_parser(%i,%s,%s);\n", type, value, from_service->name);

    while ((to = st_dup_next_word(&value)))
    {
        service_db_set_another_string(type, from_service, to);
        st_jump_spaces(&value);
    }
    return (TRUE);
}

static int parse_xml_opt(char *name, char *value, service_h * srv)
{
    int i;

    for (i = 0; option_xml_table[i].opt_name != NULL; i++)
    {
        if (strcasecmp(option_xml_table[i].opt_name, name) == 0)
        {

            D_("parse_xml_opt(%s,%s): at option no %i.\n", name, value, i);
            return (option_xml_table[i].
                    parse_func(option_xml_table[i].type, value, srv));
        }

    }

    F_("parse_xml_opt(%s): error line: \"%s\"!\n", name, value);
    F_("sleeping 2 seconds......\n");
    sleep(2);
    return (FALSE);
}

/*
   static int is_true(char *value) {
   while(value[0]==' ' || value[0]=='=') value++;
   if(strncasecmp(value,"on",2)==0 ||
   strncasecmp(value,"yes",3)==0 ||
   strncasecmp(value,"true",4)==0)
   return(TRUE);
   return(FALSE);
   }
 */

static int is_false(char *value)
{
    while (value[0] == ' ' || value[0] == '=')
        value++;
    if (strncasecmp(value, "no", 2) == 0 ||
        strncasecmp(value, "off", 3) == 0 ||
        strncasecmp(value, "false", 5) == 0)
        return (TRUE);
    return (FALSE);
}

static service_h *initng_parse_service_ezxml(ezxml_t node, char *watch_for,
                                             service_h * father, e_type type)
{
    char *name = NULL;
    service_h *copy = NULL;
    service_h *new_service = NULL;
    service_h *service_to_return = NULL;

    assert(watch_for);
    if (!node)
        return (NULL);

    D_("initng_parse_service_ezxml();\n");

    if (!node->attr[0] ||
        !strcasecmp(node->attr[0], "name") == 0 ||
        !(node->attr[1]) || !(name = strdup(node->attr[1])))
    {
        F_("initng_parse_service_ezxml(): service must have a name!");
        return (NULL);
    }

    D_("initng_parse_service_ezxml(): service name: \"%s\"\n", name);

    /* create a service entry */
    if (node->attr[2] &&
        strcasecmp(node->attr[2], "father") == 0 && node->attr[3])
    {
        copy = service_db_find_by_name((char *) node->attr[3]);
        if (copy)
            service_db_copy(&new_service, name, copy);
    }
    else
    {
        service_db_new(&new_service, name, type);
    }

    if (!new_service)
    {
        F_("initng_parse_service_ezxml(): Unable to allocate service.\n");
        return (NULL);
    }

    /* set type if not set. */
    if (father)
        new_service->type = type;

    node = node->child;

    while (node)
    {
        service_h *tmp = NULL;

        /*
           if(strcmp((char *)node->name,"text")==0) {
           node=node->next;    
           continue;
           }
         */
        D_("node-name:%s\n", node->name);
        if (strcasecmp("service", (char *) node->name) == 0)
            if ((tmp =
                 initng_parse_service_ezxml(node->child, watch_for, NULL,
                                            TYPE_SERVICE)))
                service_to_return = tmp;

        if (strcasecmp("class", (char *) node->name) == 0)
            if ((tmp =
                 initng_parse_service_ezxml(node->child, watch_for, NULL,
                                            TYPE_CLASS)))
                service_to_return = tmp;

        /* parse line - search for keywords in option_table */
        parse_xml_opt((char *) node->name, node->txt, new_service);

        /* go to next node */
        node = node->ordered;
    }

    /* add to service db */
    if (!service_db_add(new_service))
    {
        F_("can't add service to service db!\n");
        if (new_service->name)
            free(new_service->name);
        if (new_service)
            free(new_service);
        return (NULL);
    }

    /* check if this is the service we ar looking for */
    if (st_service_cmp(watch_for, new_service->name))
        return (new_service);

    /* if we got a service from a under service */
    if (service_to_return)
        return (service_to_return);
    /* return null if not */
    return (NULL);

}

/* parse a file for services */
static service_h *initng_parse_ezxml_file(char *filename, char *watch_for)
{
    service_h *service_to_return = NULL;
    service_h *tmp = NULL;
    ezxml_t xml_file;

    assert(filename);
    assert(watch_for);
    D_("parse_xml_file(%s);\n", filename);

    /* open xml file */
    if (!(xml_file = ezxml_parse_file(filename)))
    {
        D_("parse_ezxml_file(%s): Can't open xml file for parsing.\n",
           filename);
        return (NULL);
    }

    /* big loop */
    while (xml_file)
    {
        D_("headnode: %s\n", xml_file->name);
        if (strcasecmp("service", xml_file->name) == 0)
        {
            if ((tmp =
                 initng_parse_service_ezxml(xml_file, watch_for, NULL,
                                            TYPE_SERVICE)))
                service_to_return = tmp;

        }
        else if (strcasecmp("class", xml_file->name) == 0)
        {
            if ((tmp =
                 initng_parse_service_ezxml(xml_file, watch_for, NULL,
                                            TYPE_CLASS)))
                service_to_return = tmp;
        }
        xml_file = xml_file->ordered;

    }

    /*exit_initng(1); */
    if (service_to_return)
        return (service_to_return);
    return (NULL);
}

/* Load a service from a process_name or process_path */
service_h *initng_ezxml_load_process(char *process_name)
{
    char filetoparse[200];
    char *tmp = NULL;
    service_h *got_serv = NULL;

    assert(process_name);
    /* example  INITNG_ROOT=/etc/initng process_name="deamon/sshd XML_EXT=".xml" */
    /* TRY NO 1, try load   "/etc/initng/daemon/sshd.xml" */
    /* TRY NO 2, try load   "/etc/initng/daemon/sshd/sshd.xml" */
    /* TRY NO 3, try load   "/etc/initng/daemon/sshd/default.xml" */
    /* TRY NO 4, try load   "/etc/initng/daemon/default.xml" */
    /* TRY NO 5, try load   "/etc/initng/daemon/daemon.xml */
    /* TRY NO 6, try load   "/etc/initng/sshd.xml" */
    /* TRY NO 7, try load   "/etc/initng/daemon.xml */

    /* TRY NO 1, try load   "/etc/initng/daemon/sshd/sshd.xml" */
    strcpy(filetoparse, INITNG_ROOT "/");
    strcat(filetoparse, process_name);
    strcat(filetoparse, "/");
    strcat(filetoparse, st_strip_path(process_name));
    strcat(filetoparse, XML_EXT);
    if ((got_serv = initng_parse_ezxml_file(filetoparse, process_name)))
        return (got_serv);

    /* TRY NO 2, try load   "/etc/initng/daemon/sshd.xml" */
    strcpy(filetoparse, INITNG_ROOT "/");
    strcat(filetoparse, process_name);
    strcat(filetoparse, XML_EXT);
    if ((got_serv = initng_parse_ezxml_file(filetoparse, process_name)))
        return (got_serv);

    /* TRY NO 3, try load   "/etc/initng/daemon/sshd/default.xml" */
    strcpy(filetoparse, INITNG_ROOT "/");
    strcat(filetoparse, process_name);
    strcat(filetoparse, "/default" XML_EXT);
    if ((got_serv = initng_parse_ezxml_file(filetoparse, process_name)))
        return (got_serv);

    /* TRY NO 4, try "/etc/initng/daemon/default.xml" */
    tmp = st_get_path(process_name);
    strcpy(filetoparse, INITNG_ROOT "/");
    strcat(filetoparse, tmp);
    strcat(filetoparse, "/default" XML_EXT);
    free(tmp);
    if ((got_serv = initng_parse_ezxml_file(filetoparse, process_name)))
        return (got_serv);

    /* TRY NO 5, try "/etc/initng/daemon/daemon.xml" */
    tmp = st_get_path(process_name);
    strcpy(filetoparse, INITNG_ROOT "/");
    strcat(filetoparse, tmp);
    strcat(filetoparse, "/");
    strcat(filetoparse, tmp);
    strcat(filetoparse, XML_EXT);
    free(tmp);
    if ((got_serv = initng_parse_ezxml_file(filetoparse, process_name)))
        return (got_serv);

    /* TRY NO 6, try "/etc/initng/sshd.xml" */
    strcpy(filetoparse, INITNG_ROOT "/");
    strcat(filetoparse, st_strip_path(process_name));
    strcat(filetoparse, XML_EXT);
    if ((got_serv = initng_parse_ezxml_file(filetoparse, process_name)))
        return (got_serv);

    /* TRY NO 7, try "/etc/initng/daemon.xml" */
    tmp = st_get_path(process_name);
    strcpy(filetoparse, INITNG_ROOT "/");
    strcat(filetoparse, tmp);
    strcat(filetoparse, XML_EXT);
    free(tmp);
    if ((got_serv = initng_parse_ezxml_file(filetoparse, process_name)))
        return (got_serv);

    D_("initng_ezxml_load_process(%s): got \"NULL\"\n", process_name);
    return (NULL);
}


void module_init(void)
{
    int i;

    D_("moule_init();\n");
    /* add initng_xml_load_process to list of parsers */
    for (i = 0; i < MAX_PARSERS; i++)
    {
        if (!g.parsers[i])
        {
            g.parsers[i] = &initng_ezxml_load_process;
            break;
        }
    }
    D_("initng_ezxml_parser.so.0.0 loaded as parser #%i !\n", i);

}

void module_unload(void)
{
    int i;

    D_("moule_unload();\n");
    /* remove initng_xml_load_process in list of parsers */
    for (i = 0; i < MAX_PARSERS; i++)
    {
        if (g.parsers[i] == &initng_ezxml_load_process)
        {
            g.parsers[i] = NULL;
            break;
        }
    }
    D_("initng_ezxml_parser.so.0.0 unloaded as parser #%i !\n", i);

}
