#include <stdlib.h>
#include <stdio.h>
#include <dlfcn.h>
#include "initng.h"
#include "service_db.h"
#include "initng_load_module.h"
#include "initng_global.h"

/*service_h *(*parsers[MAX_PARSERS]) (char *to_parse); */
/*void *modules[MAX_MODULES]; */

/* this will null all arrays */
void initng_load_module_init(void)
{
    int i;

    for (i = 0; i < MAX_MODULES; i++)
        g.modules[i] = NULL;

    for (i = 0; i < MAX_PARSERS; i++)
        g.parsers[i] = NULL;

    for (i = 0; i < MAX_STATE_WATCHERS; i++)
        g.state_watchers[i] = NULL;
}

/* this will dlclose all arrays */
void initng_load_module_close_all(void)
{
    int i;
    char *error;
    void *(*unload_mod) ();

    for (i = 0; i < MAX_MODULES; i++)
        if (g.modules[i])
        {

            unload_mod = dlsym(g.modules[i], "module_unload");

            if ((error = dlerror()) != NULL)
            {
                F_("dlerror: %s\n\n", error);
            }

            if (unload_mod)
                (*unload_mod) ();

            dlclose(g.modules[i]);
        }
}

service_h *initng_modules_parse(char *name)
{
    int i;
    service_h *to_ret;

    for (i = 0; i < MAX_PARSERS; i++)
    {
        if (g.parsers[i])
        {
            to_ret = (*g.parsers[i]) (name);
            if (to_ret)
                return (to_ret);
        }
    }
    return (NULL);
}

/* called when a service status has changed. */
void initng_load_module_state_changed(active_h * service)
{
    int i;

    for (i = 0; i < MAX_STATE_WATCHERS; i++)
        if (g.state_watchers[i])
            (*g.state_watchers[i]) (service);
    return;
}

/* load a module= a parser module for now */
int initng_load_module(char *module_name)
{
    int i;
    char *error;
    void *(*init_mod) ();

    /* find a free entry */
    for (i = 0; i < MAX_MODULES; i++)
        if (!g.modules[i])
            break;

    g.modules[i] = dlopen(module_name, RTLD_LAZY);

    if (!g.modules[i])
    {
        F_("%s", dlerror());
        return (FALSE);
    }

    D_("Succeed to open lib \"%s\"\n", module_name);

    init_mod = dlsym(g.modules[i], "module_init");

    if ((error = dlerror()) != NULL)
    {
        F_("dlerror: %s\n\n", error);
    }

    if (init_mod)
        (*init_mod) ();

    return (TRUE);
}
