
/* Initng, a next generation sysvinit replacement.
 * Copyright (C) 2005 Jimmy Wennlund <jimmy.wennlund@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "initng.h"
#define _GNU_SOURCE
#include <stdio.h>
#include <alloca.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdlib.h>
#include "service_db.h"
#include "string_tools.h"
#include "initng_rl_parser.h"
#include "initng_global.h"
#include "open_read_close.h"

/* a simple parser for a runlevel file */
service_h *initng_rl_parser(char *runlevel_name)
{
    service_h *n_service;       /* service struct pointer too   */
    char filetoparse[200];
    char *w_depends = NULL;
    char *w = NULL;
    char *a = NULL;

    strcpy(filetoparse, INITNG_ROOT "/");
    strcat(filetoparse, runlevel_name);
    strcat(filetoparse, ".runlevel");
    D_("initng_rl_parser(%s): loading file %s.\n",
       runlevel_name, filetoparse);

    /* allocate a new service */
    service_db_new(&n_service, strdup(runlevel_name), TYPE_RUNLEVEL);

    /* copy file filename into allocated file_buf */
    if (!open_read_close(filetoparse, &w_depends))
    {
        D_("parse_file(%s): Cant open config file!\n", filetoparse);
        if (n_service->name)
            free(n_service->name);
        if (n_service)
            free(n_service);
        return (NULL);
    }
    w = w_depends;

    st_jump_to_valid_line(&w);
    while (w[0] != '\0')
    {
        a = st_dup_line(&w);
        D_("adding dep: \"%s\"\n", a);
        service_db_set_another_string(STRING_NEED, n_service, a);
        st_jump_to_valid_line(&w);
    }

    free(w_depends);
    service_db_set(ENABLED_UP_WHEN_DONE, n_service);
    service_db_set_string(STRING_FROM_FILE, n_service, strdup(filetoparse));

    return (service_db_add(n_service));
}

void module_init()
{
    int i;

    D_("moule_init();\n");
    /* add initng_rl_parser to list of parsers */
    for (i = 0; i < MAX_PARSERS; i++)
    {
        if (!g.parsers[i])
        {
            g.parsers[i] = &initng_rl_parser;
            break;
        }
    }
    D_("initng_rl_parser.so.0.0 loaded as parser #%i !\n", i);

}

void module_unload()
{
    int i;

    D_("moule_unload();\n");
    /* removing initng_rl_parser in list of parsers */
    for (i = 0; i < MAX_PARSERS; i++)
    {
        if (g.parsers[i] == &initng_rl_parser)
        {
            g.parsers[i] = NULL;
            break;
        }
    }
    D_("initng_rl_parser.so.0.0 unloaded as parser #%i !\n", i);

}
